//============================================================================
// CFA-10006-12864C Demonstration code
// LOW_LCD.C: Low-level lcd display routines.
// RA6963 LCD Controller
// Atmel ATMega2561 processor @ 16MHz
// WinAVR / AVR GCC + Atmel AVR Studio
// Written by Brian Lilly
// Copyright 2005-2014, Crystalfontz America, Inc. http://www.crystalfontz.com
//============================================================================
#include <avr/io.h>
#include <util/delay.h>
#include <avr/iom2561.h>
#include <avr/pgmspace.h>
#include <string.h>
#include "utils.h"
#include "low_lcd.h"
#include "CFAG16080C1_Splash.h"
//============================================================================
//This is the display memory.
volatile unsigned char display[1600];  //1024 bytes

//============================================================================
volatile unsigned char display_status;

//============================================================================
void write_command(unsigned char command)
{
	PORTA=command;// write the command to the port

	CLR_CE;       // chip selected
	SET_CD;       // set to write to the control registers
	SET_RD;       // writing 
	SET_WR;       // clock in the command

	_delay_us(200);
	CLR_WR;       // bring write line low
	_delay_us(200);
	SET_WR;       // clock in the command
	_delay_us(200);

	SET_CE;       // unselect chip
}

//============================================================================
// PORTC0_LCD_CS1 and PORTD6_LCD_CS2 must be set to select the desired
// controller
//----------------------------------------------------------------------------
void write_data(unsigned char data)
{
	PORTA=data;// write the command to the port

	CLR_CE;       // chip selected
	CLR_CD;       // set to write to the control registers
	SET_RD;       // writing
	SET_WR;       // clock in the command

	_delay_us(200);
	CLR_WR;       // bring write line low
	_delay_us(200);
	SET_WR;       // clock in the command
	_delay_us(200);

	SET_CE;       // unselect chip
}


//=============================================================================
// Takes about 5.08ms (201.7KB/S) reading busy flag
void UpdateLCD(void)
{
	memcpy_P(display, boot_logo, sizeof(display));
	write_data(0x00);
	write_data(0x08);
	write_command(0x24);

	write_command(0xB0);  // auto write
	int x=0;

	for(int row=0;row<80;row++)
	{
		for(int col=0;col<20;col++)
		{
			write_data(display[x]);
			x++;
		}
	}
	write_command(0xB2);  // auto write reset
}
//=============================================================================
void ResetLCD(void)
{
     CLR_RESET;
     _delay_us(6500);
     SET_RESET;
}
//=============================================================================
void ClearText()
{
	int i = 8192;

	write_data(0x00);
	write_data(0x00);
	write_command(0x24);

	write_command(0xB0);  // auto write

	while(i--)
	{
		write_data(0x00);
	}
	write_command(0xB2);  // auto write reset
}
//=============================================================================
void ClearDisplay()
{
	int i = 8192;

	write_data(0x00);
	write_data(0x08);
	write_command(0x24);

	write_command(0xB0);  // auto write

	while(i--)
	{
		write_data(0x00);
	}

	write_command(0xB2);  // auto write reset
}
//=============================================================================
void InitializeLCD(void)
{
	SET_RD;
	SET_WR;
	CLR_CD;
	SET_CE;
	PORTA=0x00;

	SET_RESET;
	_delay_ms(10);
	CLR_RESET;
	_delay_ms(100);
	SET_RESET;
	_delay_ms(500);

	write_data(0x00);
	write_data(0x00);
	write_command(0x40);	// set text home address

	write_data(0x14); 		// 160/8
	write_data(0x00); 
	write_command(0x41);	// set text area

	write_data(0x00);
	write_data(0x08); 		// address 0x0800
	write_command(0x42);	// set graphic home address

	write_data(0x14);
	write_data(0x00);
	write_command(0x43);	// set graphic area

	write_command(0xA0);	// 1 line cursor
	write_command(0x81);	// mode set - exor mode
	write_command(0x9C);	// display mode - graphic on, text on

	write_data(0x00);
	write_data(0x00);
	write_command(0x21);	// set cursor position 

	//Clear the display memeory
	memset(display, 0x00, sizeof(display));
}
//=============================================================================
